#ifndef ARCH_H
#define ARCH_H

#include <string.h>

#define ARCH_HAS_GOROUD_H
/* Type Definitions */

typedef unsigned int   unsigned_32bit;
typedef signed int     signed_32bit;
typedef unsigned short unsigned_16bit;
typedef signed short   signed_16bit;
typedef unsigned char  byte;

typedef signed_32bit   fixedpoint;
typedef float	       real;

#ifdef __GNUC__
#define HAS_INLINE
#define INLINE __inline__
#define CONST __const__
#else
#define INLINE
#define CONST
#endif

/* Memory Functions: */
#define HW_MemSet(ptr,chr,sze) memset(ptr,chr,sze*sizeof(byte))
#define HW_MemCpy(dst,src,sze) memcpy(dst,src,sze)

/* Fixed Point functions: */
extern INLINE CONST fixedpoint FpMult(fixedpoint a,fixedpoint b)
#ifdef HAS_INLINE
{
  register long res;

  __asm__ __volatile__ (" movl %1,%%eax \n" /* source a */
       " imull %2 \n"     /* multiply with b %eax*b=> %edx(high)+%eax(low) */
       " addl $32768,%%eax \n" /* round */
       " adcl $0,%%edx \n"      /* round */
       " shrdl $16,%%edx,%%eax \n" /* shift down */
       " movl %%eax,%0 \n"     /* store result */
       : /* output */ "=g" (res) 
       : /* input */ "%g" (a) , "g" (b)
       : /* clobbered*/ "cc","%eax","%edx"  );
       
  return res;
}
#else
;
#endif

extern INLINE CONST fixedpoint FpDiv(fixedpoint a,fixedpoint b)
#ifdef HAS_INLINE
{
  register long res;
  __asm__ __volatile__ (
       " xorl %%eax,%%eax \n"
       " movl %1,%%edx \n" /* source a */
       " movl %%edx,%%ebx \n"
       " shrdl $16,%%ebx,%%eax \n"
       " sarl  $16,%%edx \n"
       " idivl %2 \n"     /* %edx(high)+%eax(low) / b */
       " movl %%eax,%0 \n"     /* store result */
       : /* output */ "=g" (res) 
       : /* input */ "g" (a) , "g" (b)
       : /* clobbered*/ "cc","%eax","%ebx","%edx"  );
       
  return res;
}
#else
;
#endif

#endif /* ARCH_H */


