#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/fbio.h>
#include <sys/file.h>
#include <sys/mman.h>
#include <memory.h>
#include <malloc.h>

#include <arch.h>
#include <device.h>

/* Externally used variables: */
byte *HW_OffscreenBuffer=0;
signed_32bit HW_ScreenSizeX;
signed_32bit HW_ScreenSizeY;
signed_32bit HW_ScreenMaxX;
signed_32bit HW_ScreenMaxY;
signed_32bit HW_ScreenCenterX;
signed_32bit HW_ScreenCenterY;
unsigned_32bit HW_BytesPerRow;

/* Internal variables: */
#define START	262144
int fd;
char *fb;
int RealWidth,RealHeight;
struct fbcmap oldcmap;
unsigned char oldr[256],oldg[256],oldb[256];

int HW_OpenScreen(void)
{
  struct fbtype fbuf;

  if ((fd = open("/dev/fb", O_RDWR)) < 0) {
    printf("Can't open '/dev/fb'\n");
    return 0;
  }
  ioctl(fd, FBIOGTYPE, &fbuf);
  RealWidth = fbuf.fb_width;
  RealHeight = fbuf.fb_height;
  fb = mmap((caddr_t) 0, RealWidth * RealHeight, PROT_READ | PROT_WRITE,
		MAP_SHARED, fd, (off_t) START);
  if (fb == (caddr_t) -1) {
    printf("Can't mmap()\n");
    return 0;
  }

  oldcmap.index=0;
  oldcmap.count=256;
  oldcmap.red=oldr;
  oldcmap.green=oldg;
  oldcmap.blue=oldb;
  if (ioctl(fd, FBIOGETCMAP, &oldcmap) < 0) {
    printf("Can't get old palette\n");
    return 0;
  }

  HW_ScreenSizeX=640;
  HW_ScreenSizeY=400;
  HW_BytesPerRow=HW_ScreenSizeX;
  HW_ScreenMaxX=HW_ScreenSizeX-1;
  HW_ScreenMaxY=HW_ScreenSizeY-1;
  HW_ScreenCenterX=(HW_ScreenSizeX/2)<<4;
  HW_ScreenCenterY=(HW_ScreenSizeY/2)<<4;
  HW_OffscreenBuffer=malloc(HW_BytesPerRow*HW_ScreenSizeY);
  if ( HW_OffscreenBuffer == 0 ) {
    printf("Can't allocate offscreen buffer\n");
    return 0;
  }

  return 1;
}

void HW_DisplayScreen(void)
{
  int i;
  char *src=HW_OffscreenBuffer;
  char *dst=fb;
  for (i=0;i<HW_ScreenSizeY;i++) {
    memcpy(dst,src,HW_ScreenSizeX);
    dst+=RealWidth;
    src+=HW_ScreenSizeX;
  }
}

void HW_SetPalette(byte *orgpal) 
{
  static struct fbcmap bgc[1];
  static unsigned char r[256];
  static unsigned char g[256];
  static unsigned char b[256];
  int i;
  for (i=0;i<256;i++) {
    r[i]=orgpal[i*3];
    g[i]=orgpal[i*3+1];
    b[i]=orgpal[i*3+2];
  }

  bgc[0].index =0;
  bgc[0].count = 256;
  bgc[0].red = r;
  bgc[0].green = g;
  bgc[0].blue = b;
  ioctl(fd, FBIOPUTCMAP, &bgc[0]);
}


void HW_CloseScreen(void)
{
  /* Restore palette */
  ioctl(fd, FBIOPUTCMAP, &oldcmap);

  /* Free memory */
  if (HW_OffscreenBuffer) {
    free(HW_OffscreenBuffer);
  }
}


