#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <vga.h>
#include <vgagl.h>

#include <arch.h>
#include <device.h>

/* Externally used variables: */
byte *HW_OffscreenBuffer=0;
signed_32bit HW_ScreenSizeX;
signed_32bit HW_ScreenSizeY;
signed_32bit HW_ScreenMaxX;
signed_32bit HW_ScreenMaxY;
signed_32bit HW_ScreenCenterX;
signed_32bit HW_ScreenCenterY;
unsigned_32bit HW_BytesPerRow;

/* Internal variables: */
static GraphicsContext visible,hidden;

int HW_OpenScreen(void)
{

  if (vga_init()) {
    fprintf(stderr,"Can't init vga\n");
    return 0;
  }


#define GMODE G320x240x256

  HW_ScreenSizeX=320;
  HW_ScreenSizeY=240;
  HW_ScreenMaxX=HW_ScreenSizeX-1;
  HW_ScreenMaxY=HW_ScreenSizeY-1;
  HW_ScreenCenterX=(HW_ScreenSizeX/2)<<4;
  HW_ScreenCenterY=(HW_ScreenSizeY/2)<<4;

  if (vga_setmode(GMODE)) {
    fprintf(stderr,"Can't init vga\n");
    return 0;
  }


  gl_setcontextvga(GMODE);
  gl_getcontext(&visible);
  gl_clearscreen(0);
  gl_enablepageflipping(&visible); 
  gl_setcontextvgavirtual(GMODE);
  gl_getcontext(&hidden);
	
  HW_OffscreenBuffer=hidden.vbuf;
  HW_BytesPerRow=hidden.bytewidth;

  return 1;
}

void HW_DisplayScreen(void)
{
  gl_copyscreen(&visible);
/*  vga_copytoplanar256(HW_OffscreenBuffer,
         HW_BytesPerRow*2,0,HW_ScreenSizeY,
         HW_ScreenSizeX,HW_ScreenSizeY);*/
}

void HW_SetPalette(byte *orgpal) 
{
  int i;
  for(i = 0; i < 256; i++) {
    vga_setpalette(i,orgpal[3*i+0]>>2,orgpal[3*i+1]>>2,orgpal[3*i+2]>>2);
  }
}

void HW_ClearOffscreen(void)
{
  HW_MemSet(HW_OffscreenBuffer, 0, HW_BytesPerRow*HW_ScreenSizeY);
}

void HW_CloseScreen(void)
{
  gl_freecontext(&hidden);
  gl_freecontext(&visible);
  vga_setmode(TEXT);
}






